package dolmen_loop

  1. Overview
  2. Docs

Module Dolmen_loop.TyperSource

Typechecker state

Sourcetype ty_state

The type of state used by the Make functor.

Sourceval new_state : unit -> ty_state

Generate a fresh typing state. The bool argument given decides whether locations are kept (provides better error messages, but uses more memory when running).

Typechecker Functor

Sourcemodule type S = Typer_intf.S
Sourcemodule Make (S : State_intf.Typer with type ty_state := ty_state) : S with type state := S.t and type ty_state := ty_state and type env := T.env and type 'a fragment := 'a T.fragment and type error := T.error and type warning := T.warning and type builtin_symbols := T.builtin_symbols

Typechecker Pipe

Sourcemodule type Pipe_arg = Typer_intf.Pipe_arg
Sourcemodule type Pipe_res = Typer_intf.Pipe_res
Sourcemodule Pipe (Expr : Expr_intf.S) (Print : Expr_intf.Print with type ty := Expr.ty and type ty_var := Expr.ty_var and type ty_cst := Expr.ty_cst and type term := Expr.term and type term_var := Expr.term_var and type term_cst := Expr.term_cst and type formula := Expr.formula) (State : State_intf.Typer_pipe) (Typer : Pipe_arg with type state := State.t and type ty := Expr.ty and type ty_var := Expr.ty_var and type ty_cst := Expr.ty_cst and type term := Expr.term and type term_var := Expr.term_var and type term_cst := Expr.term_cst and type formula := Expr.formula) : Pipe_res with type state := State.t and type ty := Expr.ty and type ty_var := Expr.ty_var and type ty_cst := Expr.ty_cst and type term := Expr.term and type term_var := Expr.term_var and type term_cst := Expr.term_cst and type formula := Expr.formula
Sourceval almost_linear : string Report.Warning.t

Almost linear warning.