package dolmen_loop

  1. Overview
  2. Docs

Module T.ConstSource

A module for constant symbols that occur in terms.

The type of constant symbols that can occur in terms

Sourceval print : Format.formatter -> t -> unit

Printing function for term constants.

Sourceval compare : t -> t -> int

Comparison function on constant symbols.

Sourceval arity : t -> int * int

Returns the arity of a term constant.

Create a constant symbol.

Sourceval set_tag : t -> 'a Tag.t -> 'a -> unit

Tag a constant.

Sourceval add_tag : t -> 'a list Tag.t -> 'a -> unit

Add a value to the list of values bound to a tag.