package dolmen_loop

  1. Overview
  2. Docs

Module Ty.ConstSource

A module for constant symbols the occur in types.

The type of constant symbols the can occur in types

Sourceval compare : t -> t -> int

Comparison function on type constants.

Sourceval print : Format.formatter -> t -> unit

Printing function.

Sourceval arity : t -> int

Return the arity of the given symbol.

Sourceval mk : Dolmen.Std.Path.t -> int -> t

Create a type constant with the given arity.

Sourceval set_tag : t -> 'a Tag.t -> 'a -> unit

Set the value bound to a tag.

Sourceval add_tag : t -> 'a list Tag.t -> 'a -> unit

Add a value to the list of values bound to a tag.