package dolmen_loop

  1. Overview
  2. Docs

Module Ty.VarSource

A module for variables that occur in types.

The type of variables the can occur in types

Sourceval compare : t -> t -> int

Comparison function on variables.

Sourceval mk : string -> t

Create a new type variable with the given name.