package dolmen_loop

  1. Overview
  2. Docs

Module Dolmen_loop.Typer_intfSource

Typer

Sourcemodule type Pipe_types = sig ... end
Sourcemodule type Pipe_arg = sig ... end

This modules defines the smallest signatures for a typechecker that allow to instantiate the Typer.Pipe functor.

Sourcemodule type S = sig ... end

This modules defines the result signature of the Typer.Make functor

Sourcemodule type Pipe_res = sig ... end

This modules defines the result signature of the Typer.Pipe functor